; File: INTTEST.ASM

; Author: Matthew Mastracci

; Purpose: Low-level graphics routines

[BITS 32]

%include "general.inc"

[GLOBAL _InitGraphics__Fv]
[GLOBAL _InitMode13h__Fv]
[GLOBAL _PutPixel__FUiUiUc]
[GLOBAL _RestoreTextMode__Fv]
[EXTERN ___dpmi_segment_to_descriptor]

[SECTION .text ALIGN=4]

; ---------------------------------------------------------------------------
; Prototype: int InitGraphics(void);
; Returns: -1 on error, else zero
; ---------------------------------------------------------------------------
align
_InitGraphics__Fv:
        push ebp
        mov ebp, esp

        push dword 0a000h                     ; Push segment for function
        call ___dpmi_segment_to_descriptor    ; Note the triple underscore
        cmp eax, -1                           ; Check for error  
        jne .SelectorOkay                     ; No error, selector is valid  

        mov word [VideoRAM], 0ffffh           ; Error, make selector invalid
        jmp .Done                             ; End function, will return -1

.SelectorOkay:
        mov [VideoRAM], ax                    ; Load selector from ax
        xor eax, eax                          ; Clear eax to return zero  

.Done:
        mov esp, ebp
        pop ebp
        ret

; ---------------------------------------------------------------------------
; Prototype: void InitMode13h(void);
; Returns: nothing
; ---------------------------------------------------------------------------
align
_InitMode13h__Fv:
        push ebp
        mov ebp, esp

        ; ah = 00h, Sets video mode with int 10h to mode al
        mov ax, 0013h
        call SaveRegs
        ; Simulate real-mode interrupt
        mov ax, 0300h
        ; Of int 10h
        mov bx, 0010h
        ; Copy nothing from the stack
        mov cx, 0000h
        ; Load es with ds, so es:edi points to RegSet
        push ds
        pop es
        mov edi, RegSet
        ; Do it
        int 31h
        ; Load results back into registers
        call RestoreRegs

        mov esp, ebp
        pop ebp
        ret

; ---------------------------------------------------------------------------
; Prototype: void PutPixel(unsigned int x, unsigned int y, unsigned char Color);
; Returns: nothing
; ---------------------------------------------------------------------------
x_PutPixel      equ 8
y_PutPixel      equ 12
Color_PutPixel  equ 16

align
_PutPixel__FUiUiUc:
        push ebp
        mov ebp, esp

        mov eax, [ebp + y_PutPixel]
        mov ebx, eax
        shl eax, 6
        shl ebx, 8
        add ebx, eax
        add ebx, [ebp + x_PutPixel]
        mov fs, [VideoRAM]
        mov al, [ebp + Color_PutPixel]
        mov [fs:ebx], al

        mov esp, ebp
        pop ebp
        ret

; ---------------------------------------------------------------------------
; Prototype: void RestoreTextMode(void);
; Returns: nothing
; ---------------------------------------------------------------------------
align
_RestoreTextMode__Fv:
        push ebp
        mov ebp, esp

        ; ah = 00h, Sets video mode with int 10h to mode al
        mov ax, 0003h
        call SaveRegs
        ; Simulate real-mode interrupt
        mov ax, 0300h
        ; Of int 10h
        mov bx, 0010h
        ; Copy nothing from the stack
        mov cx, 0000h
        ; Load es with ds, so es:edi points to RegSet
        push ds
        pop es
        mov edi, RegSet
        ; Do it
        int 31h
        ; Load results back into registers
        call RestoreRegs

        mov esp, ebp
        pop ebp
        ret

SaveRegs:
        mov [SaveEAX], eax
        mov [SaveEBX], ebx
        mov [SaveECX], ecx
        mov [SaveEDX], edx
        mov [SaveESI], esi
        mov [SaveEDI], edi 
        mov [SaveEBP], ebp
        pushf
        pop eax
        mov [SaveFlags], ax
        ret

RestoreRegs:
        pushf
        pop eax
        mov ax, [SaveFlags]
        push eax
        popf
        mov eax, [SaveEAX]
        mov ebx, [SaveEBX]
        mov ecx, [SaveECX]
        mov edx, [SaveEDX]
        mov esi, [SaveESI]
        mov edi, [SaveEDI]
        mov ebp, [SaveEBP]
        ret

[SECTION .data ALIGN=4]

; Register set structure for int 31h, function 3000h
RegSet
SaveEDI         dd 00000000
SaveESI         dd 00000000
SaveEBP         dd 00000000
                dd 00000000
SaveEBX         dd 00000000
SaveEDX         dd 00000000
SaveECX         dd 00000000
SaveEAX         dd 00000000
SaveFlags       dw 0000
SaveES          dw 0000       
SaveDS          dw 0000       
SaveFS          dw 0000       
SaveGS          dw 0000       
SaveIP          dw 0000       
SaveCS          dw 0000       
SaveSP          dw 0000       
SaveSS          dw 0000       

align
VideoRAM        dw 0000h
