/*
File: RMCBTEST.CC

Author: Matthew Mastracci

Purpose: Allocates a real-mode callback wrapper, passes it to the resident 
mouse driver and then frees it.
*/

#include <go32.h>
#include <dos.h>
#include <dpmi.h>
#include <conio.h>
#include <stdio.h>
#include <crt0.h>

int _CRT0_STARTUP_FLAGS = _CRT0_FLAG_LOCK_MEMORY;

/* To make sure the name doesn't get mangled */
extern "C" void MouseCallback(_go32_dpmi_registers * r);

extern volatile int MouseButtons;
extern volatile int MouseX;
extern volatile int MouseY;

_go32_dpmi_registers regs;

int main(void)
{
  clrscr();

  _go32_dpmi_seginfo info;

  /* Set up the handler */
  info.pm_offset = (int)MouseCallback;

  _go32_dpmi_allocate_real_mode_callback_retf(&info, &regs);

  __dpmi_regs r;

  /* Set the horizontal range valid from 0 to 1000 */
  r.x.ax = 0x07;
  r.x.cx = 0;
  r.x.dx = 1000;
  __dpmi_int(0x33, &r);

  /* Set the vertical range valid from 0 to 1000 */
  r.x.ax = 0x08;
  r.x.cx = 0;
  r.x.dx = 1000;
  __dpmi_int(0x33, &r);
  
  /* Install the real-mode callback routine */
  r.x.ax = 0x0c;
  r.x.cx = 0x1f;        /* 0x1f traps on movements and RMB/LMB presses */
  r.x.dx = info.rm_offset;
  r.x.es = info.rm_segment;
  __dpmi_int(0x33, &r);

  while (!MouseButtons) {
    printf("(%i, %i)\n", MouseX, MouseY);
    delay(250);
  }

  /* Clean up handler */
  r.x.ax = 0x0c;
  r.x.cx = 0;
  r.x.dx = 0;
  r.x.es = 0;
  __dpmi_int(0x33, &r);

  _go32_dpmi_free_real_mode_callback(&info);

  printf("Mouse button pressed.");

  return 0;
}
